package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.model.DictionaryBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 字典信息控制器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/dictionaries")
@Api(value = "字典信息管理", tags = "字典信息管理")
public class DictController {
    private final DictService dictService;

    public DictController(DictService dictService) {
        this.dictService = dictService;
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询字典树列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<List<Dictionary>> list(@RequestParam(value = "pid", required = false) Long pid) {
        return dictService.list(pid);
    }

    /**
     * 根据字典值获取子节点列表
     *
     * @param value
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询系统字典子信息", httpMethod = "GET")
    @GetMapping("/children")
    public BocloudResult<List<Dictionary>> children(@RequestParam(value = Common.VALUE, required = false) String value) {
        return dictService.listChild(value);
    }

    /**
     * 查询字典树
     *
     * @param value
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询字典树", httpMethod = "GET")
    @GetMapping("/children/tree")
    public BocloudResult<List<Dictionary>> childTree(@RequestParam(value = Common.VALUE, required = false) String value) {
        return dictService.childTree(value);
    }

    /**
     * 创建
     *
     * @param dictionary
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "新增字典", httpMethod = "POST")
    @PostMapping
    public BocloudResult<Dictionary> create(@RequestBody Dictionary dictionary) {
        return dictService.create(dictionary);
    }

    /**
     * 删除字典
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除字典", httpMethod = "DELETE")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return dictService.remove(id);
    }

    /**
     * 修改字典
     *
     * @param id
     * @param bean
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改字典", httpMethod = "PUT")
    @PutMapping("/{id}")
    public BocloudResult<Dictionary> modify(@PathVariable(value = Common.ID) Long id,
                                            @RequestBody DictionaryBean bean) {
        bean.setId(id);
        return dictService.modify(bean);
    }

}
