package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Label;
import com.bocloud.sms.interfaces.LabelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author zhangdi
 * @since 2018/7/2
 */
@RestController
@RequestMapping("/v1/tags")
@Api(value = "标签管理", tags = "标签管理")
public class LabelController {

    private final LabelService labelService;

    public LabelController(LabelService labelService) {
        this.labelService = labelService;
    }

    /**
     * 获取标签列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取命名规则列表", httpMethod = "GET")
    @GetMapping
    public BocloudResult<GridBean<Label>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return labelService.list(pager, context);
    }

    /**
     * 添加标签
     *
     * @param label
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "添加标签", httpMethod = "POST")
    @PostMapping
    public BocloudResult<Label> create(@RequestBody Label label, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return labelService.create(label,context);
    }

    /**
     * 修改标签
     *
     * @param id
     * @param label
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改标签", httpMethod = "PUT")
    @PutMapping("/{id}")
    public BocloudResult<Label> modify(@PathVariable(value = Common.ID) Long id, @RequestBody Label label) {
        label.setId(id);
        return labelService.modify(label);
    }

    /**
     * 删除标签
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "删除标签", httpMethod = "DELETE")
    @DeleteMapping("/{id}")
    public Result  remove(@PathVariable(value = Common.ID) Long id) {
        return labelService.remove(id);
    }

    /**
     * 查看标签详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查看标签详情", httpMethod = "GET")
    @GetMapping("/{id}")
    public BocloudResult<Label> detail(@PathVariable(value = Common.ID) Long id) {
        return labelService.detail(id);
    }
}