package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.booter.websocket.model.LogViewMessage;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 当日日志和历史日志下载
 *
 * @author zhangdi
 * @since 2018.7.23
 */
@Slf4j
@RestController
@RequestMapping("/v1/logs")
@Api(value = "日志下载管理", tags = "日志下载管理")
public class LogDownloadController {

    private static final Logger logger = LoggerFactory.getLogger(LogDownloadController.class);

    @Value("${logging.dir}")
    private String loggingDir;

    /**
     * 实时日志下载
     *
     * @param token
     * @param server
     * @param host
     * @param request
     * @param response
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "当日日志下载", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/download", method = {RequestMethod.GET})
    public void download(@RequestParam(value = "token") String token,
                         @RequestParam(value = "server") String server,
                         @RequestParam(value = "host") String host,
                         HttpServletRequest request, HttpServletResponse response) {
        Long position = 0L;
        Long presize = Long.MAX_VALUE;
        String fileName = server + ".log";
        String filepath = loggingDir + File.separator + fileName;
        LogViewMessage logViewMessage;
        switch (server) {
            case "worker": {
                String workPath = "/var/log/bocloud/bocloud_worker.log";
                String url = "http://" + host + "/api/" + server + "/v1/logview";
                logViewMessage = getRemoteLog(url, workPath, position, presize, host.substring(0, host.indexOf(':')));
                break;
            }
            case "pontus": {
                String workPath = "/var/log/pontus/pontus.log";
                String url = "http://" + host + "/api/" + server + "/v1/logview";
                logViewMessage = getRemoteLog(url, workPath, position, presize, null);
                break;
            }
            default: {
                String url = "http://" + host + "/v1/logview";
                log.info("token:{}", token);
                logViewMessage = getRemoteLog(token, url, filepath, position, presize);
                break;
            }
        }
        String content;
        try {
            content = JSONObject.toJSONString(logViewMessage.getContent());
            // 获取当前时间并拼成yyyy-MM-dd格式 HH:mm
            Date date = new Date();
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String beginIndex = dateFormat.format(date).substring(0, 11) + "00:00";
            String endIndex = dateFormat.format(date);

            // 将开始结束时间放入日历类 操作日历截取日志
            Calendar beginTime = Calendar.getInstance();
            Calendar endTime = Calendar.getInstance();
            beginTime.setTime(dateFormat.parse(beginIndex));
            // beginTime.add(Calendar.MINUTE, -1);
            endTime.setTime(dateFormat.parse(endIndex));

            // 根据开始时间和结束时间进行索引
            int begin = content.indexOf(beginIndex);
            int end = content.indexOf(endIndex);
            // 判断开始时间是否存在，不存在则向后推一分钟
            int i = 0;
            while (begin == -1) {
                beginTime.add(Calendar.MINUTE, 1);
                SimpleDateFormat beginFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String newBeginIndex = beginFormat.format(beginTime.getTime());
                begin = content.indexOf(newBeginIndex);
                i++;
                if (i > 1440) {
                    break;
                }
            }

            // 判断结束时间是否存在，不存在则向前推一分钟
            int j = 0;
            while (end == -1) {
                endTime.add(Calendar.MINUTE, -1);
                SimpleDateFormat endFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String newEndIndex = endFormat.format(endTime.getTime());
                // 索引最后一次出现的结束时间
                end = content.lastIndexOf(newEndIndex);
                j++;
                if (j > 1440) {
                    break;
                }
            }

            // 判断此时间段内是否有日志
            String total;
            if (begin > end || begin == -1) {
                total = "此时间段内无日志";
            } else {
                String substring = content.substring(begin, end);
                // 补上最后时刻的一条数据
                String character = "\\n";
                // 从end的索引开始，向后拿到到“\n”为址的第一条数据
                int endAdd = content.indexOf(character, end);
                String add = content.substring(end, endAdd);
                total = substring + add;
                // 将捕获日志时拿到的“\n”、“\\”及着色符替换掉
                total = total.replace("\\\\", "").replace("\\n", "\r\n").replace("\\u001B[36m", "")
                        .replace("\\u001B[34m", "").replace("\\u001B[31m", "").replace("\\u001B[1;31m", "")
                        .replace("\\u001B[0;39m", "").replace("\\tat", "            ").replace("\\t...", "...");
            }
            response.setContentType("text/plain");
            String filename = URLEncoder.encode(fileName, "utf-8");
            final String userAgent = request.getHeader("USER-AGENT");
            if (userAgent.contains("Mozilla")) {
                filename = new String(fileName.getBytes(), "ISO8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
            // 定义输出流
            OutputStream outputStream = response.getOutputStream();
            byte[] decode = total.getBytes();
            outputStream.write(decode);
            outputStream.close();
        } catch (Exception e) {
            logger.error("Daily log download failed!");
        }

    }

    /**
     * 历史日志打包下载
     *
     * @param server
     * @param host
     * @param begin
     * @param end
     * @param request
     * @param response
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "历史日志下载", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/zip", method = {RequestMethod.GET})
    public void zip(@RequestParam(value = "token") String token,
                    @RequestParam(value = "server") String server,
                    @RequestParam(value = "host") String host,
                    @RequestParam(value = "begin") String begin,
                    @RequestParam(value = "end") String end,
                    HttpServletRequest request, HttpServletResponse response) {
        try {
            String filepath = loggingDir + File.separator + "history";
            // 定义导出文件名称
            Map<String, byte[]> map;
            String fileName = server + ".zip";
            if ("cmc".equals(server)) {
                map = getLocalZip(filepath, begin, end, server);
            } else {
                String url = "http://" + host + "/v1/logzip";
                map = getRemoteZip(token, url, filepath, begin, end, server);
            }

            response.setContentType("application/x-zip-compressed");
            String filename = URLEncoder.encode(fileName, "utf-8");
            final String userAgent = request.getHeader("USER-AGENT");
            if (userAgent.contains("Mozilla")) {
                filename = new String(fileName.getBytes(), "ISO8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + filename);
            // 创建一个临时文件
            File file = new File(System.getProperty("user.home") + File.separator + fileName);
            OutputStream outputStream = response.getOutputStream();
            try (ZipOutputStream zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
                byte[] buffer = new byte[1024];
                // 遍历拿到的每个文件的数据
                for (Map.Entry<String, byte[]> entry : map.entrySet()) {
                    InputStream inputStream = new ByteArrayInputStream(entry.getValue());
                    // 文件命名
                    zipOutputStream.putNextEntry(new ZipEntry(entry.getKey()));
                    int len;
                    // 读入需要下载的文件的内容
                    while ((len = inputStream.read(buffer)) > 0) {
                        zipOutputStream.write(buffer, 0, len);
                    }
                    zipOutputStream.closeEntry();
                }
            } catch (IOException e) {
                logger.error("Get error message：", e);
            }

            try (FileInputStream fileInputStream = new FileInputStream(file)) {
                IOUtils.copy(fileInputStream, outputStream);
                outputStream.close();
            } catch (IOException e) {
                logger.error("Get error message：", e);
            }
            // 删除临时文件
            boolean flag = file.delete();
            logger.info("临时文件删除状态：{}", flag);
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
    }

    /**
     * 获取远端服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @return
     */
    private LogViewMessage getRemoteLog(String token, String url, String filepath, Long position, Long presize) {
        LogViewMessage message;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        Result result = client.get(MapTools.simpleMap("token", token), param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            if (null != jsonObject) {
                jsonObject = JSONObject.parseObject(jsonObject.getString("message"));
            } else {
                jsonObject = new JSONObject();
            }
            message = new LogViewMessage(jsonObject.getString("content"), jsonObject.getLong("position"));
        }
        return message;
    }

    /**
     * 获取远端worker服务器上的日志文件内容
     *
     * @param url      服务地址
     * @param filepath 文件路径
     * @param position 起始位点
     * @param presize  前移位点
     * @param target   worker服务器IP地址
     * @return
     */
    private LogViewMessage getRemoteLog(String url, String filepath, Long position, Long presize, String target) {
        LogViewMessage message;
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("position", position);
        param.put("presize", presize);
        if (StringUtils.isNotEmpty(target)) {
            param.put("target", target);
        }
        Result result = client.post(null, param, url);
        if (result.isFailed()) {
            message = new LogViewMessage("", position);
        } else {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            message = new LogViewMessage(jsonObject.getString("message"), jsonObject.getLong("filesize"));
        }
        return message;
    }

    /**
     * 获取cmc本地的历史日志
     *
     * @param filepath
     * @param begin
     * @param end
     * @param server
     * @return
     */
    private Map<String, byte[]> getLocalZip(String filepath, String begin, String end, String server) {

        List<File> files = new ArrayList<>();
        File file = new File(filepath);
        File[] tempList = file.listFiles();
        if (null != tempList && tempList.length > 0) {
            for (File value : tempList) {
                if (value.isFile()) {
                    files.add(value);
                }
            }
        }
        List<File> list = new ArrayList<>();
        for (File newFile : files) {
            if (newFile.getName().contains(server)) {
                // 判断文件名长度是否有14位，所需的文件长度都大于14位
                if (newFile.getName().length() < 14) {
                    continue;
                }
                // 截取文件名字中的第4-14个字符 ，拿到日期
                String date = newFile.getName().substring(4, 14);
                if (date.compareTo(begin) >= 0 && date.compareTo(end) <= 0) {
                    list.add(newFile);
                }
            }
        }
        Map<String, byte[]> dataMap = Maps.newHashMap();
        byte[] data;
        for (File newFile : list) {
            try (FileInputStream fis = new FileInputStream(newFile);
                 ByteArrayOutputStream bos = new ByteArrayOutputStream()) {
                byte[] buffer = new byte[1024];
                int len;
                while ((len = fis.read(buffer)) != -1) {
                    bos.write(buffer, 0, len);
                }
                data = bos.toByteArray();
                dataMap.put(newFile.getName(), data);
            } catch (IOException e) {
                logger.error("Get error message：", e);
            }
        }
        return dataMap;
    }

    /**
     * 获取远程服务器历史日志
     *
     * @param url
     * @param filepath
     * @param begin
     * @param end
     * @param server
     * @return
     */
    private Map<String, byte[]> getRemoteZip(String token, String url, String filepath, String begin, String end,
                                             String server) {
        HttpClient client = new HttpClient();
        client.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> param = MapTools.simpleMap("filepath", filepath);
        param.put("begin", begin);
        param.put("end", end);
        param.put("server", server);
        Result result = client.get(MapTools.simpleMap("token", token), param, url);

        Map<String, byte[]> dataMap = Maps.newHashMap();
        if (result.isSuccess() && StringUtils.isNotEmpty(result.getMessage())) {
            JSONObject object = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            for (String key : object.keySet()) {
                dataMap.put(key, object.getBytes(key));
            }
        }
        return dataMap;
    }
}
