package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.Message;
import com.bocloud.sms.interfaces.MessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/messages")
@Api(value = "消息管理", tags = "消息管理")
public class MessageController {

    private final MessageService messageService;

    public MessageController(MessageService messageService) {
        this.messageService = messageService;
    }

    /**
     * 获取消息列表
     *
     * @param pager   分页对象
     * @param context 请求对象
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取消息列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<Message>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return messageService.list(pager, context);
    }

    /**
     * 删除消息
     *
     * @param id      消息id
     * @param context 请求对象
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除消息", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return messageService.remove(id, context.getTarget());
    }

    /**
     * 消息详情
     *
     * @param id      消息id
     * @param context 请求对象
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取消息详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<Message> detail(@PathVariable(value = Common.ID) Long id,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return messageService.detail(id, context);
    }

    /**
     * 当前用户消息状态全部更新为已读
     *
     * @param context 请求对象
     * @return
     */
    @PatchMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "更改消息状态", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public Result readAll(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return messageService.messageRead(context);
    }

    /**
     * 统计当前用户消息的状态数量
     *
     * @param context 请求对象
     * @return
     */
    @GetMapping(value = "/stats")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "统计消息数量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<Message> statistics(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return messageService.statistics(context);
    }
}
