package com.bocloud.sms.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.sms.entity.SystemNotice;
import com.bocloud.sms.interfaces.SystemNoticeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 *
 */
@RestController
@RequestMapping("/v1/system/notice")
@Api(value = "系统公告管理", tags = "系统公告管理")
public class SystemNoticeController {

    private final SystemNoticeService service;

    public SystemNoticeController(SystemNoticeService systemNoticeService) {
        this.service = systemNoticeService;
    }


    /**
     * 创建
     * @param entity
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "创建", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody SystemNotice entity, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.create(entity,requestContext);
    }

    /**
     * 删除
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.remove(id,requestContext);
    }

    /**
     * 修改
     * @param id
     * @param entity
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody SystemNotice entity,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.modify(entity,requestContext);
    }

    /**
     * 发布
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "发布", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/send/{id}")
    public Result send(@PathVariable(value = Common.ID) Long id,
                       @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.sendMsg(id);
    }

    /**
     * 列表
     * @param pager
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<SystemNotice>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.list(pager, requestContext);
    }

    /**
     * 详情
     * @param id
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<SystemNotice> detail(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return service.detail(id);
    }
}
