/**
 * @author hty
 * @time 上午9:39:37
 * @date 2014年12月5日
 */
package com.bocloud.sms.booter.websocket.message;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.http.server.ServletServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;

import com.bocloud.cmp.boot.core.utils.Tokens;
import com.bocloud.common.utils.Common;

import io.jsonwebtoken.Claims;

@Component
public class MessageInterceptor implements HandshakeInterceptor {

	private final String SEC_WEBSOCKET_PROTOCOL = "Sec-WebSocket-Protocol";
	private static Logger logger = LoggerFactory.getLogger(MessageInterceptor.class);

	@Override
	public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler,
			Map<String, Object> attributes) throws Exception {
		if (request instanceof ServletServerHttpRequest) {
			HttpServletRequest httpRequest = ((ServletServerHttpRequest) request).getServletRequest();
			String token = httpRequest.getHeader("Sec-WebSocket-Protocol");
			if (!StringUtils.hasText(token)) {
				return false;
			}
			Claims claims = Tokens.parse(token);
			if (null == claims) {
				return false;
			}
			String catalog = claims.get("catalog", String.class);
			if (!StringUtils.hasText(catalog)) {
				return false;
			}
			Long id = claims.get(Common.UUID, Long.class);
			if (null == id) {
				return false;
			}
			attributes.put("catalog", catalog);
			attributes.put("id", id);
			return true;
		}
		return false;
	}

	@Override
	public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler,
			Exception exception) {
		if (request instanceof ServletServerHttpRequest) {
			HttpServletRequest servletRequest = ((ServletServerHttpRequest) request).getServletRequest();
			HttpServletResponse servletResponse = ((ServletServerHttpResponse) response).getServletResponse();
			if (StringUtils.hasText(servletRequest.getHeader(SEC_WEBSOCKET_PROTOCOL))) {
				servletResponse.addHeader("Sec-WebSocket-Protocol", servletRequest.getHeader("Sec-WebSocket-Protocol"));
			}
		}
	}
}