package com.bocloud.sms.interfaces;

import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.sms.entity.Department;
import com.bocloud.sms.entity.Manager;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.model.DepartmentModel;

import java.util.List;

/**
 * 组织机构抽象Service接口
 *
 * @author tyl
 */
public interface DepartmentService {

    /**
     * 创建组织机构
     *
     * @param params
     * @param requestContext
     *
     * @throws InternalServerException
     * @return
     */
    Result create(DepartmentModel params, RequestContext requestContext) throws InternalServerException;

    /**
     * 修改组织机构
     *
     * @param id
     * @param params
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result modify(Long id, DepartmentModel params, RequestContext requestContext) throws InternalServerException;

    /**
     * 删除组织机构
     *
     * @param id
     * @param requestContext
     * @throws InternalServerException
     * @return
     */
    Result remove(Long id, RequestContext requestContext) throws InternalServerException;

    /**
     * 组织机构详情
     *
     * @param id
     * @return
     */
    BocloudResult<Department> detail(Long id);

    /**
     * 查询组织机构
     *
     * @param parentId
     * @param requestContext
     * @return
     */
    BocloudResult<List<Department>> list(Long parentId, RequestContext requestContext);

    /**
     * 根据层级查询组织机构
     *
     * @param parentId
     * @param requestContext
     * @return
     */
    BocloudResult<List<Department>> listByParentId(Long parentId, RequestContext requestContext);

    /**
     * 是否存在
     *
     * @param name
     * @param tenantId
     * @return
     */
    BocloudResult<Department> exists(String name, Long tenantId);

    /**
     * 查询组织机构下管理员
     *
     * @param id
     * @param requestContext
     * @return
     */
    BocloudResult<List<User>> listDeptManager(Long id, RequestContext requestContext);
}
