package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.enums.BaseStatus;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.*;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.license.LicenseDaemon;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.service.registry.ServiceClient;
import com.bocloud.sms.entity.CloudVendor;
import com.bocloud.sms.enums.CloudProvider;
import com.bocloud.sms.interfaces.CloudVendorService;
import com.bocloud.sms.model.Butler;
import com.bocloud.sms.model.CloudVendorBean;
import com.bocloud.sms.model.PowervcConstant;
import com.bocloud.sms.repository.CloudVendorRepository;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 云账号业务类
 */
@Service("cloudVendorService")
public class CloudVendorServiceImpl implements CloudVendorService, InitializingBean {

    private final LicenseDaemon licenseDaemon;
    private Logger logger = LoggerFactory.getLogger(CloudVendorServiceImpl.class);
    @Value("${platform.sync.schedule:0 0 0 * * ?}")
    private String defaultExpression;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private ServiceClient serviceClient;

    public CloudVendorServiceImpl(CurrentService currentService) {
        this.licenseDaemon = LicenseDaemon.instance(currentService);
    }

    @Override
    public BocloudResult<CloudVendor> create(CloudVendor vendor, RequestContext context) {
        try {
            // 先进行同名认证，虽然输入时已验证，可能浏览器会记录信息
            CloudVendor existedVendor = cloudVendorRepository.getByName(vendor.getName());
            Assert.isNull(existedVendor, "添加失败，名称重复");
            String type = vendor.getType();
            if (type != null) {
                type = type.toUpperCase();
            }
//            existedVendor = cloudVendorRepository.getByRegion(vendor.getRegion(), vendor.getAvailablitiyZone(), type);
//            Assert.notNull(existedVendor, "当前可用域已包含该类型云平台！");
            // 云平台认证信息
            JSONObject authentication = JSON.parseObject(vendor.getAuthentication());
            Assert.notNull(authentication, "平台认证信息参数有误");
            if (CloudProvider.OPENSTACK.name().equals(type) || CloudProvider.VMWARE.name().equals(type)
                    || CloudProvider.ALIYUN.name().equals(type) || CloudProvider.JDCLOUD.name().equals(type)
                    || CloudProvider.H3C.name().equals(type) || CloudProvider.HMC.name().equals(type)
                    || CloudProvider.FUSIONCLOUD.name().equals(type) || CloudProvider.MANAGEONE.name().equals(type)
                    || CloudProvider.TENCENT.name().equals(type) || CloudProvider.HUAWEI.name().equals(type)
                    || CloudProvider.XEN.name().equals(type) || CloudProvider.POWERVC.name().equals(type)
                    || CloudProvider.CLOUDOS.name().equals(type) || CloudProvider.AZURE.name().equals(type)
                    || CloudProvider.KINGCLOUD.name().equals(type) || CloudProvider.KUBERNETES.name().equals(type)
                    || CloudProvider.AWS.name().equals(type) || CloudProvider.QCLOUD.name().equals(type)
                    || CloudProvider.TCE.name().equals(type) || CloudProvider.PQCLOUD.name().equals(type)
                    || CloudProvider.TIANYI.name().equals(type)) {
                // 节点数校验
                int currentNode = currentService.getUsed();// 当前节点数
                int node = licenseDaemon.content().getNode();
                if (currentNode > node) {
                    return new BocloudResult<>(false, "当前系统节点数（宿主机和云主机）已超过license限制，云平台添加失败!", "500", "");
                }
            }
            if (!CloudProvider.JDCLOUD.name().equals(type) && !CloudProvider.ALIYUN.name().equals(type)
                    && !CloudProvider.TENCENT.name().equals(type) && !CloudProvider.HUAWEI.name().equals(type)
                    && !CloudProvider.XEN.name().equals(type) && !CloudProvider.KINGCLOUD.name().equals(type)
                    && !CloudProvider.AZURE.name().equals(type) && !CloudProvider.KUBERNETES.name().equals(type)
                    && !CloudProvider.AWS.name().equals(type) && !CloudProvider.QCLOUD.name().equals(type)
                    && !CloudProvider.TCE.name().equals(type) && !CloudProvider.PQCLOUD.name().equals(type)
                    && !CloudProvider.F5.name().equals(type) && !CloudProvider.TIANYI.name().equals(type)) {
                // 正则获取IP
                String address = authentication.getString("address");
                String regex = "\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}";
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(address);
                if (m.find()) {
                    address = m.group();
                }
                if (address.endsWith("/")) {
                    address = address.substring(0, address.length() - 1);
                }
                existedVendor = cloudVendorRepository.getByAddress(address);
                JSONObject oldAuthentication = new JSONObject();
                if (null != existedVendor) {
                    oldAuthentication = JSON.parseObject(existedVendor.getAuthentication());
                }
                if (null != existedVendor
                        && authentication.getString("account").equals(oldAuthentication.getString("account"))) {
                    return new BocloudResult<>(false, "此平台已存在，不能重复添加!");
                }
                // 前端传过来的参数用AES解密
                Encryptor encryptor = new AESEncrptor();
                String password = encryptor.decrypt(authentication.getString("password").trim(), null);
                encryptor = new DESEncrptor();
                password = encryptor.encrypt(password, "BocloudCMP");
                authentication.put("password", password);
                if (CloudProvider.FUSIONCLOUD.name().equals(type)) {
                    authentication.put("projectName", authentication.getString("projectId"));
                }
            }
            if (CloudProvider.HUAWEI.name().equals(type)) {
                Encryptor encryptor = new AESEncrptor();
                String password = encryptor.decrypt(authentication.getString("password").trim(), null);
                encryptor = new DESEncrptor();
                password = encryptor.encrypt(password, "BocloudCMP");
                authentication.put("password", password);
                String regions = authentication.getString("regions");
                List<String> currentRegions = JSON.parseArray(regions, String.class);
                if (ListTool.isEmpty(currentRegions)) {
                    return new BocloudResult<>(false, "区域信息错误！");
                }
                authentication.put("regionId", currentRegions.get(0));
            } else if (CloudProvider.F5.name().equals(type)) {
                Encryptor encryptor = new AESEncrptor();
                String password = encryptor.decrypt(authentication.getString("password").trim(), null);
                encryptor = new DESEncrptor();
                password = encryptor.encrypt(password, "BocloudCMP");
                authentication.put("password", password);
            } else if (CloudProvider.TCE.name().equals(type)) {
                String regionId = authentication.getString("regionId");
                List<String> regions = new ArrayList<>();
                regions.add(regionId);
                authentication.put("regions", JSON.toJSONString(regions));
            } else if (CloudProvider.POWERVC.name().equals(type)) {
                String projectName = authentication.getString("projectName");
                if (StringUtils.isEmpty(projectName)) {
                    authentication.put("projectName", PowervcConstant.DEFAULT_PROJECT_NAME);
                }
                String domainName = authentication.getString("domainName");
                if (StringUtils.isEmpty(domainName)) {
                    authentication.put("domainName", PowervcConstant.DEFAULT_DOMAIN_NAME);
                }
            }
            if (CloudProvider.TCE.name().equals(type) || CloudProvider.ALIYUN.name().equals(type)
                    || CloudProvider.TENCENT.name().equals(type) || CloudProvider.QCLOUD.name().equals(type)
                    || CloudProvider.PQCLOUD.name().equals(type) || CloudProvider.KINGCLOUD.name().equals(type)
                    || CloudProvider.AWS.name().equals(type) || CloudProvider.AZURE.name().equals(type)
                    || CloudProvider.TIANYI.name().equals(type) || CloudProvider.HUAWEI.name().equals(type)) {
                if (authentication.containsKey("accessKey")) {
                    Encryptor encryptor = new AESEncrptor();
                    String accessKey = encryptor.decrypt(authentication.getString("accessKey").trim(), null);
                    encryptor = new DESEncrptor();
                    accessKey = encryptor.encrypt(accessKey, "BocloudCMP");
                    authentication.put("accessKey", accessKey);
                }
                if (authentication.containsKey("secretKey")) {
                    Encryptor encryptor = new AESEncrptor();
                    String secretKey = encryptor.decrypt(authentication.getString("secretKey").trim(), null);
                    encryptor = new DESEncrptor();
                    secretKey = encryptor.encrypt(secretKey, "BocloudCMP");
                    authentication.put("secretKey", secretKey);
                }
                String regions = authentication.getString("regions");
                List<String> currentRegions = JSON.parseArray(regions, String.class);
                if (!ListTool.isEmpty(currentRegions)) {
                    authentication.put("regionId", currentRegions.get(0));
                }
            }
            // 重新设置封装后的认证信息
            vendor.setAuthentication(JSON.toJSONString(authentication));
            vendor.setStatus(BaseStatus.NORMAL.name());
            vendor.setUuid(KeyGenerator.uuid());
            // 在zk注册信息
            List<CloudVendor> list = cloudVendorRepository.listAll();
            list.add(vendor);
            this.updateProvider(list);
            vendor.setId(null);
            if (StringUtils.isEmpty(vendor.getSyncCron())) {
                vendor.setSyncCron(defaultExpression);
            }
            cloudVendorRepository.save(vendor);
            return new BocloudResult<>(true, vendor, "平台添加成功，正在同步数据中...");
        } catch (Exception e) {
            logger.error("create cloudVendor fail:", e);
            return new BocloudResult<>(false, "平台添加失败：" + e.getMessage());
        }
    }


    @Transactional
    @Override
    public BocloudResult<CloudVendor> modify(CloudVendorBean bean, Long userId) {
        String path = CloudVendor.class.getSimpleName() + "_" + bean.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudVendor vendor = cloudVendorRepository.query(bean.getId());
            Assert.notNull(vendor, "数据不存在");
            if (!bean.getName().equals(vendor.getName())) {
                CloudVendor object = cloudVendorRepository.getByName(bean.getName());
                Assert.isNull(object, "名称不能重复!");
            }
            String availablitiyZone = bean.getAvailablitiyZone();
            if (null != availablitiyZone && !availablitiyZone.equals(vendor.getAvailablitiyZone())) {
                CloudVendor object = cloudVendorRepository.getByRegion(bean.getRegion(), bean.getAvailablitiyZone(),
                        CloudProvider.VMWARE.name());
                Assert.isNull(object, "当前可用域下已有其他云平台");
            }
            if (CloudProvider.KUBERNETES.name().equals(vendor.getType())) {
                Long oldHarborId = vendor.getHarborId();
                Long newHarborId = bean.getHarborId();
                if (oldHarborId.equals(newHarborId)) {
                    CloudVendor cloudVendor = cloudVendorRepository.queryByBarborId(newHarborId);
                    Assert.isNull(cloudVendor, "当前仓库已被绑定，请选择其他仓库！");
                }
            }
            BeanUtils.copyProperties(bean, vendor);
            vendor.setMenderId(userId);
            Boolean result = cloudVendorRepository.update(vendor);
            if (result) {
                // 在zk刷新注册信息
                List<CloudVendor> list = cloudVendorRepository.listAll();
                this.updateProvider(list);
            }
            return result ? new BocloudResult<>(true, vendor, "修改成功")
                    : new BocloudResult<>(false, vendor, "修改失败");
        } catch (Exception e) {
            logger.error("modify cloudVendor fail:", e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Transactional
    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = CloudVendor.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudVendor vendor = cloudVendorRepository.query(id);
            Assert.notNull(vendor, "数据不存在");
            result = cloudVendorRepository.delete(id, context.getTarget());
            // 在zk注册信息
            List<CloudVendor> cloudVendors = cloudVendorRepository.listAll();
            this.updateProvider(cloudVendors);
        } catch (Exception e) {
            logger.error("remove cloudVendor fail:", e);
            return new BocloudResult<>(false, "删除失败");
        }
        return result ? new BocloudResult<>(true, "删除成功") : new BocloudResult<>(false, "删除失败");
    }

    @Transactional
    @Override
    public BocloudResult lock(Long id, Long userId) {
        String path = CloudVendor.class.getSimpleName() + "_" + id;
        boolean result;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudVendor vendor = cloudVendorRepository.query(id);
            Assert.notNull(vendor, "数据不存在");
            result = cloudVendorRepository.lock(id, userId);
        } catch (Exception e) {
            logger.error("freeze cloudVendor fail:", e);
            return new BocloudResult<>(false, "冻结失败");
        }
        return result ? new BocloudResult<>(true, "冻结成功") : new BocloudResult<>(false, "冻结失败");
    }

    @Transactional
    @Override
    public BocloudResult active(Long id, Long userId) {
        String path = CloudVendor.class.getSimpleName() + "_" + id;
        boolean result;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudVendor vendor = cloudVendorRepository.query(id);
            Assert.notNull(vendor, "数据不存在");
            result = cloudVendorRepository.active(id, userId);
        } catch (Exception e) {
            logger.error("freeze cloudVendor fail:", e);
            return new BocloudResult<>(false, "激活失败");
        }
        return result ? new BocloudResult<>(true, "激活成功") : new BocloudResult<>(false, "激活失败");
    }

    @Override
    public BocloudResult<CloudVendor> detail(Long id) {
        try {
            CloudVendor vendor = cloudVendorRepository.query(id);
            return new BocloudResult<>(true, vendor, "查询详情成功");
        } catch (Exception e) {
            logger.error("query cloudVendor detail fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult checkName(String name) {
        try {
            CloudVendor vendor = cloudVendorRepository.getByName(name);
            return null == vendor ? new BocloudResult<>(true, "检查名称成功！") : new BocloudResult<>(false, "检查名称重复!");
        } catch (Exception e) {
            logger.error("get cloudVendor by name fail", e);
            return new BocloudResult<>(false, "检查名称失败!");
        }
    }

    /**
     * @since 2018-05-22 modify by sxp 支持hmc平台信息的查询
     */
    @Override
    public BocloudResult<GridBean<CloudVendor>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            int page = Optional.ofNullable(pager.getPage()).orElse(1);
            int rows = Optional.ofNullable(pager.getRows()).orElse(10);
            List<CloudVendor> list;
            List<SimpleBean> beans;
            int total;
            GridBean<CloudVendor> gridBean;
            total = cloudVendorRepository.count(params);
            if (simple) {
                beans = this.cloudVendorRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = cloudVendorRepository.list(page, rows, params, sorter);
                gridBean = GridHelper.getBean(page, rows, total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list cloudVendor fail: {}", e.toString());
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private BocloudResult<List<CloudVendor>> listAllVendors() {
        try {
            List<Param> params = new ArrayList<>();
            return new BocloudResult<>(true, cloudVendorRepository.list(params), "查询成功！");
        } catch (Exception e) {
            logger.error("查询全部云平台异常：{}", e.getMessage());
            return new BocloudResult<>(false, "查询全部云平台异常：" + e.getMessage());
        }
    }

    @Override
    public List<CloudVendor> list(String type) {
        try {
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("type", type), Sign.EQ));
            return cloudVendorRepository.list(1, Integer.MAX_VALUE, params, null, null);
        } catch (Exception e) {
            logger.error("list cloudVendor fail:", e);
            return Lists.newArrayList();
        }
    }

    @Override
    public List<CloudVendor> listAll() {
        try {
            return cloudVendorRepository.listAll();
        } catch (Exception e) {
            logger.error("list cloudVendor fail:", e);
            return Lists.newArrayList();
        }
    }

    @Override
    public BocloudResult listAuthProvider(List<String> types, Long tenantId) {
        try {
            List<CloudVendor> vendors = cloudVendorRepository.listAuthProvider(types, tenantId);
            return new BocloudResult<>(true, vendors, "查询成功");
        } catch (Exception e) {
            logger.error("list cloudVendor fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }


    /**
     * 根据平台类型查询所有平台
     *
     * @param type
     * @return
     */
    @Override
    public BocloudResult getByType(String type) {
        try {
            List<CloudVendor> cloudVendors = cloudVendorRepository.listByType(type);
            return new BocloudResult<>(true, cloudVendors, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listByPool(String params) {
        try {
            JSONObject parseObject = JSON.parseObject(params, JSONObject.class);
            String category = parseObject.getString("category");
            if (null == category) {
                return new BocloudResult<>(false, "参数格式有误！");
            }
            Boolean isPublic = null;
            if (parseObject.containsKey("isPublic")) {
                isPublic = parseObject.getBoolean("isPublic");
            }
            List<CloudVendor> cloudVendors = cloudVendorRepository.listByPool(category, isPublic);
            return new BocloudResult<>(true, cloudVendors, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult batchOpenAlarm(List<Long> ids, Long userId) {
        if (null == ids) {
            return ResultTools.formatErrResult();
        }
        try {
            List<CloudVendor> succIds = Lists.newArrayList();
            int succResult = 0;
            int failResult = 0;
            for (Long id : ids) {
                CloudVendor cloudVendor = cloudVendorRepository.query(id);
                cloudVendor.setIsAlarm(true);
                cloudVendor.setGmtModify(new Date());
                cloudVendor.setMenderId(userId);
                boolean result = cloudVendorRepository.update(cloudVendor);
                if (result) {
                    succIds.add(cloudVendor);
                    succResult++;
                } else {
                    failResult++;
                }
            }
            return new BocloudResult<>(true, succIds, "批量开启告警成功【" + succResult + "】个，失败【" + failResult + "】个");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "批量开启告警失败");
        }
    }

    @Override
    public BocloudResult batchCloseAlarm(List<Long> ids, Long userId) {
        if (ListTool.isEmpty(ids)) {
            return new BocloudResult<>(false, "参数格式错误");
        }
        try {
            List<CloudVendor> succIds = Lists.newArrayList();
            int succResult = 0;
            int failResult = 0;
            for (Long id : ids) {
                CloudVendor cloudVendor = cloudVendorRepository.query(id);
                cloudVendor.setIsAlarm(false);
                cloudVendor.setGmtModify(new Date());
                cloudVendor.setMenderId(userId);
                boolean result = cloudVendorRepository.update(cloudVendor);
                if (result) {
                    succIds.add(cloudVendor);
                    succResult++;
                } else {
                    failResult++;
                }
            }
            return new BocloudResult<>(true, succIds, "批量关闭告警成功【" + succResult + "】个，失败【" + failResult + "】个");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "批量关闭告警失败");
        }
    }

    @Override
    public BocloudResult batchOperationAlarm(List<Long> ids, String action, Long userId) {
        if ("open".equalsIgnoreCase(action)) {
            return this.batchOpenAlarm(ids, userId);
        } else {
            return this.batchCloseAlarm(ids, userId);
        }
    }

    /**
     * 根据平台类型查询
     *
     * @param types
     * @return
     */
    @Override
    public BocloudResult listByTypes(List<String> types) {
        try {
            List<CloudVendor> vendors = cloudVendorRepository.listByTypes(types, null);
            return new BocloudResult<>(true, vendors, "查询成功");
        } catch (Exception e) {
            logger.error("list cloudVendor fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    private BocloudResult<List<CloudVendor>> listByTypes(JSONObject object) {
        try {
            List<String> vendorTypes = JSON.parseArray(object.getString("vendorTypes"), String.class);
            Long dcId = object.getLong("dcId");
            List<CloudVendor> vendors = cloudVendorRepository.listByTypes(vendorTypes, dcId);
            return new BocloudResult<>(true, vendors, "查询成功");
        } catch (Exception e) {
            logger.error("list cloudVendor fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 云平台操作
     *
     * @param id
     * @param action
     * @param context
     * @return
     */
    @Override
    @Transactional
    public BocloudResult action(Long id, String action, RequestContext context) {
        switch (action) {
            case "active":
                return this.active(id, context.getTarget());
            case "lock":
                return this.lock(id, context.getTarget());
            default:
                return new BocloudResult<>(false, "暂不支持");
        }
    }

    @Override
    public BocloudResult updatePassword(Long id, CloudVendor cloudVendor, RequestContext context) {
        String path = CloudVendor.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Assert.notNull(cloudVendor, "参数格式错误");
            CloudVendor vendor = cloudVendorRepository.query(id);
            Assert.notNull(vendor, "云平台不存在");
            Encryptor encryptor = new AESEncrptor();
            String oldPassword = cloudVendor.getOldPassword();
            String newPassword = cloudVendor.getNewPassword();
            JSONObject oldAuthentication = JSON.parseObject(vendor.getAuthentication());
            if (!StringUtils.isEmpty(oldPassword) && !StringUtils.isEmpty(newPassword)) {
                oldPassword = encryptor.decrypt(oldPassword, null);
                newPassword = encryptor.decrypt(newPassword, null);
                if (oldAuthentication.containsKey("password")) {
                    encryptor = new DESEncrptor();
                    String password = encryptor.decrypt(oldAuthentication.getString("password").trim(), "BocloudCMP");
                    if (!password.equalsIgnoreCase(oldPassword)) {
                        return new BocloudResult<>(false, "原密码错误");
                    }
                    oldAuthentication.put("password", encryptor.encrypt(newPassword, "BocloudCMP"));
                    vendor.setAuthentication(oldAuthentication.toJSONString());
                    vendor.setMenderId(context.getTarget());
                    cloudVendorRepository.update(vendor);
                    // 在zk注册信息
                    List<CloudVendor> list = cloudVendorRepository.listAll();
                    this.updateProvider(list);
                }
            } else {
                if (CloudProvider.AZURE.name().equals(vendor.getType())) {
                    encryptor = new AESEncrptor();
                    String secretKey = encryptor.decrypt(cloudVendor.getSecretKey().trim(), null);
                    encryptor = new DESEncrptor();
                    secretKey = encryptor.encrypt(secretKey, "BocloudCMP");
                    oldAuthentication.put("clientId", cloudVendor.getClientId());
                    oldAuthentication.put("tenant", cloudVendor.getTenant());
                    oldAuthentication.put("secretKey", secretKey);
                } else {
                    encryptor = new AESEncrptor();
                    String accessKey = encryptor.decrypt(cloudVendor.getAccessKey().trim(), null);
                    String secretKey = encryptor.decrypt(cloudVendor.getSecretKey().trim(), null);
                    logger.info("accessKey : {}", accessKey);
                    logger.info("secretKey : {}", secretKey);
                    encryptor = new DESEncrptor();
                    accessKey = encryptor.encrypt(accessKey, "BocloudCMP");
                    secretKey = encryptor.encrypt(secretKey, "BocloudCMP");
                    oldAuthentication.put("accessKey", accessKey);
                    oldAuthentication.put("secretKey", secretKey);
                }
            }
            vendor.setAuthentication(oldAuthentication.toJSONString());
            vendor.setMenderId(context.getTarget());
            cloudVendorRepository.update(vendor);
            // 在zk注册信息
            List<CloudVendor> list = cloudVendorRepository.listAll();
            this.updateProvider(list);
            return new BocloudResult<>(true, "密码更新成功");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult condition(String condition, RequestContext context) {
        try {
            condition = Optional.ofNullable(condition).orElse("");
            JSONObject object = JSON.parseObject(condition);
            if ("checkName".equalsIgnoreCase(object.getString("condition"))) {
                return this.checkName(object.getString("name"));
            } else if ("listByType".equalsIgnoreCase(object.getString("condition"))) {
                return this.getByType(object.getString("type"));
            } else if ("listByTypes".equalsIgnoreCase(object.getString("condition"))) {
                return listByTypes(JSON.parseArray(object.getString("types"), String.class));
            } else if ("vendorTypes".equalsIgnoreCase(object.getString("condition"))) {
                return this.listByTypes(object);
            } else if ("listByPool".equalsIgnoreCase(object.getString("condition"))) {
                return this.listByPool(condition);
            } else if ("listAllVendors".equalsIgnoreCase(object.getString("condition"))) {
                return this.listAllVendors();
            } else if ("listByAz".equalsIgnoreCase(object.getString("condition"))) {
                CloudVendor vendor = cloudVendorRepository.getByRegion(object.getString("region"),
                        object.getString("zone"), CloudProvider.VMWARE.name());
                if (null == vendor) {
                    return new BocloudResult<>(false, "云平台不存在");
                }
                return new BocloudResult<>(true, vendor, "查询成功");
            } else if ("getNsxVendor".equalsIgnoreCase(object.getString("condition"))) {
                CloudVendor vendor = cloudVendorRepository.getNsxVendor(object.getString("region"),
                        object.getString("az"));
                if (null == vendor) {
                    return new BocloudResult<>(true, "查询成功");
                }
            } else if ("listType".equals(object.getString("condition"))) {
                String category = object.getString("resourceCategory");
                Set<String> types;
                if (!StringUtils.isEmpty(category)) {
                    types = cloudVendorRepository.listAll().stream().filter(cloudVendor -> category.equals(cloudVendor.getVendorCategory())).map(CloudVendor::getType).collect(Collectors.toSet());
                } else {
                    types = cloudVendorRepository.listAll().stream().map(CloudVendor::getType).collect(Collectors.toSet());
                }

                return new BocloudResult<>(true, types, "查询成功");
            }
        } catch (Exception e) {
            logger.error("查询失败：{}", e.getMessage());
        }
        return new BocloudResult<>(false, "参数格式错误");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.serviceClient.putButler(null);
        List<CloudVendor> providers = listAll();
        Map<String, String> map = this.convertProviders(providers);
        String data = JSON.parseObject(JSON.toJSONString(map)).toString();
        this.serviceClient.putButler(data);
    }

    public void updateProvider(List<CloudVendor> providers) throws Exception {
        Map<String, String> map = convertProviders(providers);
        String data = JSON.parseObject(JSON.toJSONString(map)).toString();
        serviceClient.putButler(data);
    }

    public Map<String, String> convertProviders(List<CloudVendor> providers) {
        Map<String, String> map = new HashMap<>();
        Butler butler = null;
        for (CloudVendor provider : providers) {
            JSONObject authentication = JSON.parseObject(provider.getAuthentication());
            String regions = authentication.getString("regions") == null ? "" : authentication.getString("regions");
            butler = new Butler(provider.getName(), provider.getUuid(), authentication.getString("account"),
                    authentication.getString("password"), authentication.getString("accessKey"),
                    authentication.getString("secretKey"), regions, authentication.getString("tenant"),
                    authentication.getString("address"), provider.getType(), authentication.getString("domainName"),
                    authentication.getString("projectName"), provider.getVersion(),
                    authentication.getString("regionId"));
            if (CloudProvider.KUBERNETES.name().equalsIgnoreCase(provider.getType())) {
                butler.setToken(authentication.getString("token"));
            } else if (CloudProvider.FUSIONCLOUD.name().equalsIgnoreCase(provider.getType())
                    || CloudProvider.MANAGEONE.name().equalsIgnoreCase(provider.getType())
                    || CloudProvider.TIANYI.name().equalsIgnoreCase(provider.getType())) {
                butler.setProjectId(authentication.getString("projectId"));
            } else if (CloudProvider.AZURE.name().equalsIgnoreCase(provider.getType())) {
                butler.setSubscriptionId(authentication.getString("subscriptionId"));
                butler.setClientId(authentication.getString("clientId"));
                butler.setVersion(authentication.getString("version"));
            } else if (CloudProvider.AWS.name().equalsIgnoreCase(provider.getType())) {
                butler.setRegionId("cn-north-1");
            } else if (CloudProvider.HUAWEI.name().equalsIgnoreCase(provider.getType())) {
                butler.setDomainId(authentication.getString("domainId"));
            }
            map.put(provider.getUuid(), butler.toString());
        }
        return map;
    }
}
