package com.bocloud.sms.service.ldap;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.Department;
//import org.springframework.ldap.filter.AndFilter;
//import org.springframework.ldap.filter.EqualsFilter;
import org.springframework.util.StringUtils;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.ArrayList;
import java.util.List;

public class LdapDeptRepository {

    private final LdapConnector connector;

    public LdapDeptRepository(LdapConnector connector) {
        super();
        this.connector = connector;
    }

    private DirContext getContext() {
        return this.connector.getConnector();
    }

    private String getBaseDn() {
        return this.connector.getBaseDn();
    }

    public boolean save(String dn, Department dept) {
        Attributes attributes = new BasicAttributes();
        BasicAttribute attribute = new BasicAttribute("objectclass");
        attribute.add("top");
        attribute.add("organizationalUnit");
        attributes.put(attribute);
        attributes.put("ou", dept.getName());
        if (StringUtils.hasText(dept.getRemark())) {
            attributes.put("description", dept.getRemark());
        }
        String name = dn + "," + this.getBaseDn();
        try {
            this.getContext().createSubcontext(name, attributes);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public boolean delete(String dn) {
        try {
            this.getContext().destroySubcontext(dn);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /*public Department query(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.OBJECT_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalUnit"));
        try {
            NamingEnumeration<SearchResult> results = this.getContext().search(dn + "," + this.getBaseDn(),
                    filter.encode(), sc);
            JSONObject department = new JSONObject();
            while (results.hasMore()) {
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("ou".equals(attribute.getID())) {
                            department.put("name", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            department.put("remark", attribute.get());
                        }
                    }
                }
            }
            return department.isEmpty() ? null : JSONObject.parseObject(department.toJSONString(), Department.class);
        } catch (NamingException e) {
            return null;
        }
    }*/

    public boolean update(String oldDn, Department oldDept, String newDn, Department newDept) {
        try {
            List<ModificationItem> items = new ArrayList<>();
            if (!oldDn.equals(newDn)) {
                this.getContext().rename(oldDn + "," + this.getBaseDn(), newDn + "," + this.getBaseDn());
                Attribute attribute = new BasicAttribute("ou", newDept.getName());
                items.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attribute));
            }
            if (StringUtils.hasText(newDept.getRemark())) {
                String description = newDept.getRemark();
                Attribute attribute = new BasicAttribute("description", description);
                items.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attribute));
            }
            if (items.isEmpty()) {
                return true;
            }
            this.getContext().modifyAttributes(newDn + "," + this.getBaseDn(),
                    items.toArray(new ModificationItem[0]));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

 /*   public List<Department> list(String dn) {
        SearchControls sc = new SearchControls();
        sc.setSearchScope(SearchControls.ONELEVEL_SCOPE);
        AndFilter filter = new AndFilter();
        filter.and(new EqualsFilter("objectClass", "organizationalUnit"));
        try {
            String base = !StringUtils.hasText(dn) ? this.getBaseDn() : dn + "," + this.getBaseDn();
            NamingEnumeration<SearchResult> results = this.getContext().search(base, filter.encode(), sc);
            List<Department> list = new ArrayList<>();
            while (results.hasMore()) {
                JSONObject department = new JSONObject();
                SearchResult result = results.next();
                Attributes attrs = result.getAttributes();
                if (attrs != null && attrs.size() > 0) {
                    NamingEnumeration<? extends Attribute> attributes = attrs.getAll();
                    while (attributes.hasMore()) {
                        Attribute attribute = attributes.next();
                        if ("ou".equals(attribute.getID())) {
                            department.put("name", attribute.get());
                        } else if ("description".equals(attribute.getID())) {
                            department.put("remark", attribute.get());
                        }
                    }
                }
                if (!department.isEmpty()) {
                    list.add(JSONObject.parseObject(department.toJSONString(), Department.class));
                }
            }
            return list;
        } catch (NamingException e) {
            return null;
        }
    }*/

}
