package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 告警
 *
 * @author zhangyf
 * @date 2021-07-19
 */
@Component
public class AlarmTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 因为有用户ID
     */
    @Autowired
    public AlarmTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "告警迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "alarm", "monitor_alarm", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;

        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMapOfUser.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "monitor_alarm", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size()) + 50;
            index++;
        }
        this.percent = 100;
        // 5.5去掉了threshold_id字段，不需要处理
        // 5.5 rule_id字段名没有变化，不过关联的表已经不是同一张表了，由之前的monitor_rule_relation的id变更为了monitor_rule的id
    }

    private static final String[] COLUMNS = new String[]{"id", "name", "gmt_create", "gmt_modify", "creator_id", "mender_id",
            "owner_id", "status", "props", "remark", "deal_result", "level", "type", "target_name", "alarm_val", "recovery_val",
            "gmt_trigger", "gmt_confirm", "confirmer", "solver", "gmt_solve", "vendor_id", "is_deleted", "event_alarm",
            "send_notice", "send_recovery", "tenant_id", "target", "os_category"
    };
}
