package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @since 2021/7/8 14:27
 */
@Component
public class BillItemTask implements UpgradeTask {
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 依赖于账单表更新 需要从账单数据里面获取归属信息（用户id租户id项目id业务id那些）
     */
    public BillItemTask(ApplicationContext context, UserTask userTask, ServiceBillTask billTask) {
        this.context = context;
        dependOn.add(userTask);
        dependOn.add(billTask);
    }

    @Override public String getName() {
        return "账单项数据迁移任务";
    }

    @Override public int getPercent() {
        return this.percent;
    }

    @Override public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * 相比较5.0版本 5.5去除了租户id字段 同时还需要处理迁移过程中的user_id变化
     */
    @Transactional(rollbackFor = Throwable.class) @Override public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        List<String> serviceBillColumns = UpgradeUtils.getColumns(this.dataSource, "bill_item");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "bill_item",
            serviceBillColumns.toArray(new String[0]), percent -> this.percent = Math.floorDiv(percent * 50, 100));
        this.percent = 70;

        // 查询5.0的项目 更新账单业务信息
        NamedParameterJdbcTemplate v50jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        NamedParameterJdbcTemplate v55jdbcTemplate =
            new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        String billSql = "select id,tenant_id,user_id,project_id,business_id from service_bill";
        List<Map<String, Object>> billResults = v50jdbcTemplate.queryForList(billSql, new HashMap<>());
        int projectIndex = 1;
        for (Map<String, Object> billResult : billResults) {
            long billId = Long.parseLong(billResult.get("id").toString());
            long tenantId = Long.parseLong(billResult.get("tenant_id").toString());
            long userId = Long.parseLong(billResult.get("user_id").toString());
            long projectId = Long.parseLong(billResult.get("project_id").toString());
            long businessId = Long.parseLong(billResult.get("business_id").toString());
            Map<String, Object> sqlParams = MapTools.simpleMap("billId", billId);
            sqlParams.put("tenantId", tenantId);
            sqlParams.put("userId", userId);
            sqlParams.put("projectId", projectId);
            sqlParams.put("businessId", businessId);
            String sqlUpdate =
                "update bill_item set business_id = :businessId,project_id = :projectId,user_id = :userId,tenant_id=:tenantId where bill_id = :billId ";
            v55jdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(projectIndex * 25, billResults.size()) + 50;
            projectIndex++;
        }
        String itemSql = "select id,sn,service_code from order_item";
        List<Map<String, Object>> orderItemResults = v50jdbcTemplate.queryForList(itemSql, new HashMap<>());
        int itemIndex = 1;
        for (Map<String, Object> itemResult : orderItemResults) {
            long itemId = Long.parseLong(itemResult.get("id").toString());
            String sn = itemResult.get("sn").toString();
            String serviceCode = itemResult.get("service_code").toString();
            Map<String, Object> sqlParams = MapTools.simpleMap("itemId", itemId);
            sqlParams.put("sn", sn);
            sqlParams.put("serviceCode", serviceCode);
            String sqlUpdate =
                "update bill_item set instance = :sn,service_code = :serviceCode where item_id = :itemId ";
            v55jdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(itemIndex * 25, orderItemResults.size()) + 75;
            itemIndex++;
        }
        this.percent = 100;
    }

}
