package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.HashMap;

/**
 * ci_icon数据迁移任务
 *
 * 5.0和5.5的数据完全一样，直接truncate5.5的数据，然后迁移5.0的数据就可以了
 *
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class CiIconTask implements UpgradeTask {

    private final ApplicationContext context;
    private DataSource dataSource;
    private int percent = 0;

    @Autowired
    public CiIconTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public String getName() {
        return "ci_icon数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate jdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        // 截断5.5数据
        jdbcTemplate.update("delete from ci_icon", new HashMap<>());
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "ci_icon", percent -> this.percent = percent);
        this.percent = 100;
    }
}
