package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * 数据存储数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-16
 */
@Component
public class DataStoreTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     * @param userTask
     *            依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public DataStoreTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "数据存储数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "data_store", COLUMNS,
            percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "data_store", "creator_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "data_store", "mender_id", id.getKey(),
                id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "data_store", "owner_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS = new String[] {"id", "name", "capacity", "free_space", "gmt_create",
        "gmt_modify", "gmt_expired", "remark", "server_id", "vdc_id", "vendor_id", "creator_id", "mender_id",
        "owner_id", "status", "props", "type", "path", "host_num", "vm_num", "template_num", "store_val", "guid",
        "catalog_id", "distributable_ratio", "is_shared", "resource_label", "is_deleted"};

    // 5.5增加字段: vc添加数据，留空
    // cluster_id
    // enabled
}
