package com.bocloud.sms.service.upgrade.tasks;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 流程定义迁移
 *
 * @author zhangyf
 * @date 2021-07-19
 */
@Component
public class FlowDefinitionTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context  spring上下文
     * @param userTask 因为有用户ID
     */
    @Autowired
    public FlowDefinitionTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "流程定义迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public void run() {
        this.percent = 0;
        Asserts.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "wf_definition", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;

        int index = 1;
        Map<Long, Long> idMapOfUser = UpgradeUtils.getIdMapOfUser(this.dataSource, this.context.getBean(DataSource.class));
        Map<Long, Long> idMapOfManager = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        NamedParameterJdbcTemplate targetJdbcTemplate = new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMapOfUser.entrySet()) {
            Map<String, Object> sqlParams = new HashMap<>();
            sqlParams.put("oldId", id.getKey());
            sqlParams.put("newId", id.getValue());
            targetJdbcTemplate.update("update wf_definition set creator_id = :newId where creator_catalog = 'User' and creator_id = :oldId", sqlParams);
            targetJdbcTemplate.update("update wf_definition set mender_id = :newId where mender_catalog = 'User' and mender_id = :oldId", sqlParams);
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size() + idMapOfManager.size()) + 50;
            index++;
        }
        for (Map.Entry<Long, Long> id : idMapOfManager.entrySet()) {
            Map<String, Object> sqlParams = new HashMap<>();
            sqlParams.put("oldId", id.getKey());
            sqlParams.put("newId", id.getValue());
            targetJdbcTemplate.update("update wf_definition set creator_id = :newId, creator_catalog = 'User' where creator_catalog = 'Manager' and creator_id = :oldId", sqlParams);
            targetJdbcTemplate.update("update wf_definition set mender_id = :newId, mender_catalog = 'User' where mender_catalog = 'Manager' and mender_id = :oldId", sqlParams);
            this.percent = Math.floorDiv(index * 50, idMapOfUser.size() + idMapOfManager.size()) + 50;
            index++;
        }

        // 流程定义中的定义的审核节点中有审批人的id需要替换为新的id，并且只处理管理端的即可，自服务的流程定义功能已废弃
        List<Map<String, Object>> list = targetJdbcTemplate.queryForList("select * from wf_definition where tenant_id = 0", new HashMap<>());
        for (Map<String, Object> map : list) {
            JSONArray graph = JSONArray.parseArray(String.valueOf(map.get("graph")));
            for (int i = 0; i < graph.size(); i++) {
                JSONObject jsonObject = graph.getJSONObject(i);
                if (null == jsonObject.getJSONArray("assignee")) {
                    continue;
                }
                List<Long> oldAssignees = jsonObject.getJSONArray("assignee").toJavaList(Long.class);
                List<Long> newAssignees = new ArrayList<>();
                oldAssignees.forEach(a -> newAssignees.add(idMapOfManager.get(a)));
                jsonObject.put("assignee", newAssignees);
            }
            targetJdbcTemplate.update("update wf_definition set graph = :graph where id = " + map.get("id"), MapTools.simpleMap("graph", graph.toJSONString()));
        }
        this.percent = 100;
        // 5.5比5.0多了一个type，非必填项，所以不用处理该字段，留null即可
    }

    private static final String[] COLUMNS = new String[]{"id", "name", "instance_url", "state", "graph", "version",
            "gmt_create", "gmt_modify", "creator_id", "creator_catalog", "mender_id", "mender_catalog", "is_deleted",
            "baseline_id", "remark", "tenant_id", "operation_remark"
    };
}
