package com.bocloud.sms.service.upgrade.tasks;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * IP地址数据迁移任务
 *
 * 新增加的字段留空，直接迁移5.0数据即可
 *
 * @author zdy
 * @date 2021-07-07
 */
@Component
public class IpTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     */
    @Autowired
    public IpTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "IP地址数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate jdbcTemplate = this.context.getBean(NamedParameterJdbcTemplate.class);
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "ip", COLUMNS,
            percent -> this.percent = percent);
        // 设置IP状态，5.0已有IP在5.5中都默认为used状态
        String updateSql = "update ip set status = 'used' where 1=1";
        jdbcTemplate.update(updateSql, new HashMap<>());
        this.percent = 100;
    }

    // 5.0字段
    private static final String[] COLUMNS = new String[] {"id", "ip", "mask", "gateway", "dns", "vendor_id",
        "resource_id", "target", "catalog", "subnet_id", "network_id", "pool_id", "gmt_create"};

    // 5.5增加字段:remark、status
    // 5.5IP地址的状态有free(空闲)、used(已使用)、lock(已锁定)、preempt(预占)
    // free:通过IP池扫描得来
    // used:被机器使用
    // lock:资源分配是锁定
    // preempt:人为操作进行锁定
    // 5.0IP地址无状态区分，且无扫描锁定功能，所以在和谭庸龙商量后决定，5.5认定原有的5.0IP状态为used
}
