package com.bocloud.sms.service.upgrade.tasks;

import java.util.*;
import java.util.stream.Collectors;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;

/**
 * 标签数据迁移任务
 *
 * 5.0标签存在初始化数据 5.5没有初始化数据 所以直接迁移5.0的数据
 *
 * @author zdy
 * @date 2021-07-09
 */
@Component
public class LabelTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context
     *            spring上下文
     */
    @Autowired
    public LabelTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public String getName() {
        return "标签数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        // 获取5.0标签表的category
        String selectCategorySql = "select category from label";
        List<Map<String, Object>> category50s = fromJdbcTemplate.queryForList(selectCategorySql, new HashMap<>());
        // category去重
        List<Object> disCategory =
            category50s.stream().map(category -> category.get("category")).distinct().collect(Collectors.toList());
        // 根据category查询content
        String selectContentSql = "select content from label where category = :category";

        // 最终insert sql
        String sqlInsert = "insert into label (" + UpgradeUtils.getColumnNameSql(Arrays.asList(INSERT_COLUMN)) + ") "
            + "values (" + UpgradeUtils.getColumnValueSql(Arrays.asList(INSERT_COLUMN)) + ") ";
        int index = 1;
        for (Object o : disCategory) {
            List<Map<String, Object>> contentsMaps =
                fromJdbcTemplate.queryForList(selectContentSql, Collections.singletonMap("category", o));
            List<Object> contents =
                contentsMaps.stream().map(content -> content.get("content")).collect(Collectors.toList());
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("name", o);
            paramMap.put("content", JSONObject.toJSONString(contents));
            paramMap.put("create_id", 1L);
            paramMap.put("personal", false);
            paramMap.put("tenant_id", 0L);
            paramMap.put("remark", null);
            targetJdbcTemplate.update(sqlInsert, paramMap);
        }
        // 计算百分比
        this.percent = Math.floorDiv(index * 100, category50s.size());
        index++;

        this.percent = 100;
    }

    // 5.5需要插入数据的字段，id不处理，自增
    private static final String[] INSERT_COLUMN =
        new String[] {"name", "content", "create_id", "personal", "remark", "tenant_id"};

    // 5.0字段: id, name, content, category

    // 5.5增加字段
    // create_id:已和谭庸龙确认，5.0标签默认为超级管理员创建，赋值1
    // personal:已和谭庸龙确认，5.0标签默认为公有标签，赋值false
    // remark:已和谭庸龙确认，存null
    // tenant_id:已和谭庸龙确认，赋值0

    // 5.5去除字段
    // category:已和谭庸龙确认，5.0的category为5.5的name，5.0相同category数据的content处理成json数组存入5.5的content
}
