package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @since 2021/7/8 14:27
 */
@Component
public class RecycledTask implements UpgradeTask {
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    public RecycledTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        dependOn.add(userTask);
    }

    @Override public String getName() {
        return "回收站数据迁移任务";
    }

    @Override public int getPercent() {
        return this.percent;
    }

    @Override public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Transactional(rollbackFor = Throwable.class) @Override public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 迁移回收站数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "recycled", RECYCLED,
            percent -> this.percent = Math.floorDiv(percent * 70, 100));
        this.percent = 70;
        // 处理userId
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "recycled", "user_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 15, idMap.size()) + 70;
            index++;
        }
        //把ownerId和userId更新一样
        NamedParameterJdbcTemplate jdbcTemplate = new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        String sqlUpdate = "update recycled a join recycled b on a.id = b.id set a.owner_id = b.user_id ";
        jdbcTemplate.update(sqlUpdate, new HashMap<>());
        this.percent = 100;
    }

    private static final String[] RECYCLED =
        new String[] {"name", "sn", "type", "resource_id", "gmt_create", "creator_id", "is_deleted", "gmt_modify",
            "mender_id", "project_id", "catalog", "tenant_id", "user_id", "spec"};
}
