package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 资源数据迁移任务
 *
 * @author luyanchao
 * @date 2021-06-09
 */
@Component
public class ResourceTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    public ResourceTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "机房数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "resource", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "resource", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "resource", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "resource", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        // 5.5增加了charset、type_id、uuid三个字段，均为堡垒机使用。5.0尚未支持堡垒机，所以留null即可
    }

    private static final String[] COLUMNS = new String[] {
            "catalog","category","creator_id","gmt_create","gmt_modify","group_id","group_ids","id","inventory",
            "is_accurated","is_authed","is_deleted","is_locked","mender_id","more_prop","name","os_category",
            "owner_id","password","private_ips","project_id","props","region_id","remark","resource_group","status",
            "target","tenant_id","username","vendor_id"
    };
}
