package com.bocloud.sms.service.upgrade.tasks;


import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 资源调度策略数据迁移
 *
 * 5.5相比于5.0修改了name和remark
 *
 * @author luyanchao
 * @date 2021-07-15
 */
@Component
public class SchedulerPolicyTask implements UpgradeTask {

    private final ApplicationContext context;
    private final List<UpgradeTask> dependOnTasks = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context spring上下文
     * @param userTask 因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public SchedulerPolicyTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOnTasks.add(userTask);
    }

    @Override
    public String getName() {
        return "资源调度策略数据迁移";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOnTasks;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 查询5.5数据
        String sqlSelect55 = "select id, name, remark from scheduler_policy where 1=1 ";
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect55, new HashMap<>());
        // 清除数据·
        String sqlTruncate = "delete from scheduler_policy";
        targetJdbcTemplate.update(sqlTruncate, new HashMap<>());
        // 导入5.0数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "scheduler_policy", percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 按5.5更新name和remark
        int index = 1;
        String sqlUpdate = "update scheduler_policy set name = :name, remark = :remark where id = :id ";
        for (Map<String, Object> v55Data : v55Datas) {
            Map<String, Object> paramUpdate = new HashMap<>();
            paramUpdate.put("name", v55Data.get("name"));
            paramUpdate.put("remark", v55Data.get("remark"));
            paramUpdate.put("id", v55Data.get("id"));
            targetJdbcTemplate.update(sqlUpdate, paramUpdate);

            this.percent = Math.floorDiv(index * 25, v55Datas.size()) + 50;
            index++;

        }
        this.percent = 75;
        // 处理creator_id、mender_id、owner_id
        index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "scheduler_policy", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "scheduler_policy", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "scheduler_policy", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 25, idMap.size()) + 75;
            index++;
        }
        this.percent = 100;
    }
}
