package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.common.utils.MapTools;
import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @since 2021/7/8 14:27
 */
@Component
public class ServiceBillTask implements UpgradeTask {
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;
    private int percent = 0;

    public ServiceBillTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        dependOn.add(userTask);
    }

    @Override public String getName() {
        return "账单数据迁移任务";
    }

    @Override public int getPercent() {
        return this.percent;
    }

    @Override public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    @Override public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * 相比较5.0版本 5.5去除了租户id字段 同时还需要处理迁移过程中的user_id变化
     */
    @Transactional(rollbackFor = Throwable.class) @Override public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        List<String> serviceBillColumns = UpgradeUtils.getColumns(this.dataSource, "service_bill");
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class), "service_bill", serviceBillColumns.toArray(new String[0]),
            percent -> this.percent = Math.floorDiv(percent * 70, 100)); this.percent = 70;
        // 处理userId
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        int index = 1;
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "service_bill", "user_id", id.getKey(),
                id.getValue());
            this.percent = Math.floorDiv(index * 15, idMap.size()) + 70;
            index++;
        }
        // 查询5.0的项目 更新账单业务信息
        NamedParameterJdbcTemplate v50jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        NamedParameterJdbcTemplate v55jdbcTemplate = new NamedParameterJdbcTemplate(this.context.getBean(DataSource.class));
        String projectSql = "select id, business_id from project where is_deleted = 0 and business_id is not null";
        List<Map<String, Object>> projectResults = v50jdbcTemplate.queryForList(projectSql, new HashMap<>());
        int projectIndex = 1;
        for (Map<String, Object> projectResult : projectResults) {
            long projectId = Long.parseLong(projectResult.get("id").toString());
            long businessId = Long.parseLong(projectResult.get("business_id").toString());
            Map<String, Object> sqlParams = MapTools.simpleMap("projectId", projectId);
            sqlParams.put("businessId", businessId);
            String sqlUpdate = "update service_bill set business_id = :businessId where project_id = :projectId ";
            v55jdbcTemplate.update(sqlUpdate, sqlParams);
            this.percent = Math.floorDiv(projectIndex * 15, projectResults.size()) + 85;
            projectIndex++;
        }
        this.percent = 100;
    }

}
