package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import org.springframework.amqp.utils.MapBuilder;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 系统设置数据迁移任务
 *
 * 按照code更新5.5中的数据
 *
 * @author luyanchao
 * @date 2021-06-09
 */
@Component
public class SystemConfigTask implements UpgradeTask {

    private final ApplicationContext context;
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     * @param context spring上下文
     */
    public SystemConfigTask(ApplicationContext context) {
        this.context = context;
    }

    @Override
    public String getName() {
        return "系统设置数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);

        String sqlSelect = "select id, code, name, value from system_config where 1=1";
        String sqlUpdate = "update system_config set value = :value where id = :id";
        List<Map<String, Object>> v50Datas = fromJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        List<Map<String, Object>> v55Datas = targetJdbcTemplate.queryForList(sqlSelect, new HashMap<>());
        for (Map<String, Object> v55Data : v55Datas) {
            Optional<Map<String, Object>> v50DataOpt = v50Datas.stream()
                    .filter(v50Data -> v50Data.get("code").equals(v55Data.get("code"))).findFirst();
            if (v50DataOpt.isPresent()) {
                long v55Id = Long.parseLong(v55Data.get("id").toString());
                String v50Value = v50DataOpt.get().get("value").toString();
                Map<String, Object> paramUpdate = new HashMap<>();
                paramUpdate.put("value", v50Value);
                paramUpdate.put("id", v55Id);
                targetJdbcTemplate.update(sqlUpdate, paramUpdate);
            }
        }
        this.percent = 100;
    }
}
