package com.bocloud.sms.service.upgrade.tasks;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import com.bocloud.sms.service.upgrade.UpgradeUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 运维任务数据迁移任务
 *
 * @author luyanchao
 * @date 2021-07-14
 */
@Component
public class TaskTask implements UpgradeTask {

    private int percent = 0;
    private final ApplicationContext context;
    private final List<UpgradeTask> dependOn = new ArrayList<>();
    private DataSource dataSource;

    /**
     * 构造方法
     * @param context spring上下文
     * @param userTask 依赖UserTask，因为有creator_id、mender_id、owner_id
     */
    @Autowired
    public TaskTask(ApplicationContext context, UserTask userTask) {
        this.context = context;
        this.dependOn.add(userTask);
    }

    @Override
    public String getName() {
        return "运维任务数据迁移任务";
    }

    @Override
    public int getPercent() {
        return percent;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        // 迁移数据
        UpgradeUtils.syncTable(this.dataSource, this.context.getBean(DataSource.class),
                "task", COLUMNS, percent -> this.percent = Math.floorDiv(percent, 2));
        this.percent = 50;
        // 处理creator_id、mender_id、owner_id
        int index = 1;
        Map<Long, Long> idMap = UpgradeUtils.getIdMapOfManager(this.dataSource, this.context.getBean(DataSource.class));
        for (Map.Entry<Long, Long> id : idMap.entrySet()) {
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task", "creator_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task", "mender_id", id.getKey(), id.getValue());
            UpgradeUtils.updateColumn(this.context.getBean(DataSource.class), "task", "owner_id", id.getKey(), id.getValue());
            this.percent = Math.floorDiv(index * 50, idMap.size()) + 50;
            index++;
        }
        this.percent = 100;
        // 5.5增加了gmt_execute字段，已和魏伟确认留null
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public List<UpgradeTask> getDependOnTasks() {
        return this.dependOn;
    }

    private static final String[] COLUMNS = new String[] {
            "id","name","status","is_deleted","remark","props","gmt_create","gmt_modify","creator_id","mender_id",
            "owner_id","uuid","code","graph","cron","has_suspend","scene_id","template_id","cos_id","ros_id"
    };
}
