package com.bocloud.sms.service.upgrade.tasks.lxyupgrade;

import com.bocloud.sms.service.upgrade.UpgradeTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.sql.DataSource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 字典数据迁移任务
 * <p>
 * 5.0 同父下，value不能重复，name可以重复
 * 5.5 同父下，value和name都能重复
 *
 * @author luyanchao
 * @date 2021-06-17
 */
@Component
public class DictionaryTask implements UpgradeTask {

    private final ApplicationContext context;
    private DataSource dataSource;
    private int percent = 0;

    /**
     * 构造方法
     *
     * @param context spring上下文
     */
    @Autowired
    public DictionaryTask(ApplicationContext context) {
        this.context = context;
    }


    @Override
    public String getName() {
        return "字典数据迁移任务";
    }

    @Override
    public int getPercent() {
        return this.percent;
    }

    @Override
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void run() {
        this.percent = 0;
        Assert.notNull(this.dataSource, "dataSource can't be null");
        NamedParameterJdbcTemplate targetJdbcTemplate = context.getBean(NamedParameterJdbcTemplate.class);
        NamedParameterJdbcTemplate fromJdbcTemplate = new NamedParameterJdbcTemplate(this.dataSource);
        // 5.5数据
        String v55SelectSql = "select name, value, pid from dictionary where 1=1";
        List<Map<String, Object>> v55Datas = fromJdbcTemplate.queryForList(v55SelectSql, new HashMap<>());
        // 5.6数据
        String v56SelectSql = "select name, value, pid from dictionary where 1=1";
        List<Map<String, Object>> v56Datas = targetJdbcTemplate.queryForList(v56SelectSql, new HashMap<>());
        // 因为5.5中干掉了pvalue增加了pid，所以只能一层一层迁移
        String insertSql = "insert into dictionary(name, value, pid, weight) values (:name, :value, :pid, :weight)";
        List<Map<String, Object>> currDatas = v55Datas.stream()
                .filter(data -> "0".equals(data.get("pid").toString())).collect(Collectors.toList());
        //添加weight字段
        currDatas.forEach(data -> data.put("weight", "1"));
        int index = 0;
        while (currDatas.size() > 0) {
            List<Map<String, Object>> childDatas = new ArrayList<>();
            for (Map<String, Object> currData : currDatas) {
                long pId = Long.parseLong(currData.get("pid").toString());
                String value = currData.get("value").toString();

                String checkExistsSql = "select id from dictionary where pid = :pid and value = :value ";
                Map<String, Object> checkExistsParam = new HashMap<>();
                checkExistsParam.put("pid", pId);
                checkExistsParam.put("value", value);
                List<Map<String, Object>> checkExistsData = targetJdbcTemplate.queryForList(checkExistsSql, checkExistsParam);
                Long currId;
                if (checkExistsData.isEmpty()) {
                    // 写5.6数据
                    KeyHolder keyHolder = new GeneratedKeyHolder();
                    SqlParameterSource parameterSource = new MapSqlParameterSource(currData);
                    targetJdbcTemplate.update(insertSql, parameterSource, keyHolder);
                    // 获取5.6的id
                    currId = Objects.requireNonNull(keyHolder.getKey()).longValue();
                } else {
                    currId = Long.valueOf(checkExistsData.get(0).get("id").toString());
                }
                // 查询子数据
                List<Map<String, Object>> currChildDatas = v55Datas.stream()
                        .filter(data -> data.get("pid").equals(currData.get("id"))).collect(Collectors.toList());
                // 设置子数据的pid
                currChildDatas.forEach(data -> data.put("pid", currId));
                // 添加到childDatas准备下一次while
                childDatas.addAll(currChildDatas);
                this.percent = Math.floorDiv(index * 100, currDatas.size());
                index++;
            }
            currDatas = childDatas;
        }
        this.percent = 100;
    }

    /**
     * 这些value存在于5.5的初始化数据中，但是不存在于5.6的初始化数据中
     */
    private static final String[] exclude = new String[]{
    };

    //5.5-->5.6
    //新增weight（权重排序）
}
