package com.bocloud.sms.service.utils;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * @Author hya
 * @Date 2022/3/9 10:10
 */
public class ImportExcelUtil {
    private static final Logger logger = LoggerFactory.getLogger(ImportExcelUtil.class);
    private static Result result = null;

    /** 获取读数据监听器
     * @param  biFunction 读取到数据后执行业务逻辑
     * @param threshold 数据量大时 在 invoke 方法里面多次读取时的边界
     * @param checked 表头，用来做模板检验
     * @param params 需要传递的其他参数
     * @return AnalysisEventListener
    */
    public static <T> AnalysisEventListener<T> getListener(BiFunction<List<T>, Map<String, Object>, Result> biFunction, int threshold, String[] checked, Map<String,Object> params) {
        LinkedList<T> list = new LinkedList<>();
        return new AnalysisEventListener<T>() {
            @Override
            public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
                super.invokeHeadMap(headMap, context);
                for (int i = 0; i < checked.length; i++) {
                    if (!headMap.get(i).equals(checked[i])) {
                        logger.error("导入模板错误");
                        result = new Result(false,"导入模板错误");
                        throw new IllegalArgumentException("导入模板错误");
                    }
                }
            }
            @Override
            public void invoke(T data, AnalysisContext context) {
                list.add(data);
            }

            @Override
            public void doAfterAllAnalysed(AnalysisContext context) {
                if (list.size() > 0) {
                    result = biFunction.apply(list,params);
                }
            }
        };
    }

    public static Result getResult(){
        return result;
    }

}
