package com.bocloud.sms.repository;

import com.bocloud.common.utils.MapTools;
import com.bocloud.orm.OrmGenericDaoImpl;
import com.bocloud.sms.entity.Department;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tyl
 * @date 2020/2/3 周一 11:44
 */
@Repository("departmentRepository")
public class DepartmentRepository extends OrmGenericDaoImpl<Department, Long> {

    public DepartmentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 删除组织机构信息
     *
     * @param id
     * @param menderId
     * @return
     * @throws Exception
     */
    public int delete(Long id, Long menderId) throws Exception {
        String sql = "update department set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params);
    }

    /**
     * 根据parentId查询指定租户ID的组织机构集合
     *
     * @param parentId
     * @return
     * @throws Exception
     */
    public List<Department> list(Long parentId) throws Exception {
        Map<String, Object> params = new HashMap<>(2);
        String sql;
        if (null == parentId || parentId == 0) {
            sql = "select * from department where (parent_id = 0 or parent_id is null) and is_deleted = 0";
        } else {
            sql = "select * from department where parent_id = :parentId and is_deleted = 0 ";
            params.put("parentId", parentId);
        }
        return this.list(Department.class, sql, params);
    }

    /**
     * 查询所有组织机构信息
     *
     * @return
     * @throws Exception
     */
    public List<Department> listAll() throws Exception {
        String sql = "select * from department where is_deleted = 0";
        return this.list(Department.class, sql);
    }

    /**
     * 根据id查询组织机构
     *
     * @param id
     * @return
     * @throws Exception
     */
    public Department query(Long id) throws Exception {
        String sql = "select * from department where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Department> list = this.list(Department.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one record in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询指定租户下组织机构是否存在
     *
     * @param name
     * @param tenantId
     * @return
     * @throws Exception
     */
    public Department exists(String name, Long tenantId) throws Exception {
        String sql = "select * from department where is_deleted = 0 and name = :name and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("tenantId", tenantId);
        List<Department> list = this.list(Department.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one record in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 获取指定租户下的组织机构集合
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public List<Department> listByTenant(Long tenantId) throws Exception {
        String sql = "select * from department where is_deleted = 0 and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Department.class, sql, params);
    }

    public List<Department> list(Long parentId, Long tenantId, String origin) throws Exception {
        Map<String, Object> params = MapTools.simpleMap("origin", origin);
        params.put("tenantId", tenantId);
        String sql;
        if (null == parentId || parentId == 0) {
            sql = "select * from department where (parent_id = 0 or parent_id is null) and is_deleted = 0 and tenant_id = :tenantId and origin = :origin";
        } else {
            sql = "select * from department where parent_id = :parentId and is_deleted = 0 and tenant_id = :tenantId and origin = :origin";
            params.put("parentId", parentId);
        }
        return this.list(Department.class, sql, params);
    }
}
