package com.bocloud.sms.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.sms.entity.Plugins;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 插件表
 *
 * @author fanzx
 * @version 5.6.0-SNAPSHOT
 * @since 2022/2/16
 */
@Repository("pluginsRepository")
public class PluginsRepository extends BasicGenericDao<Plugins, Long> {

    public PluginsRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 查询plugins总数
     * @param params
     * @param context
     * @return
     */
    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from plugins where is_deleted = 0 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询plugins列表
     * @param pager
     * @param context
     * @return
     */
    public List<Plugins> list(Pager pager, RequestContext context) {
        String sql = "select * from plugins where is_deleted = 0 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(pager.getParams());
        sql = QueryBuilder.buildRaw(sql, pager, "");
        return this.list(Plugins.class, sql, paramMap);
    }

    /**
     * 根据id删除plugins
     * @param id
     * @return
     */
    public boolean deleted(Long id) {
        String sql = "update plugins set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }

    /**
     * 根据插件名称 获取到插件的详细信息
     * @param plugins
     * @return
     */
    public List<Plugins> listByCode(List<String> plugins) {
        String sql = "select * from plugins where is_deleted = 0 and code in (:plugins) and status = 1 ";
        Map<String, Object> paramMap = MapTools.simpleMap("plugins", plugins);
        return this.list(Plugins.class, sql, paramMap);
    }

    /**
     * 根据插件唯一Code进行查询
     * @param code
     * @return
     */
    public Plugins getByCode(String code) {
        String sql = "select * from plugins where is_deleted = 0 and code = :code";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        List<Plugins> plugins = this.list(Plugins.class, sql, paramMap);
        return ListTool.isEmpty(plugins) ? null : plugins.get(0);
    }

    /**
     * 根据插件name进行查询
     * @param name
     * @return
     */
    public Plugins getByName(String name) {
        String sql = "select * from plugins where is_deleted = 0 and name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<Plugins> plugins = this.list(Plugins.class, sql, paramMap);
        return ListTool.isEmpty(plugins) ? null : plugins.get(0);
    }

    /**
     * 判断插件是否存在同名
     * @param name  插件名称
     * @return
     */
    public boolean existsSameName(String name) {
        String sql = "select count(1) from plugins where name = :name and is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }

    /**
     * 判断插件编码是否重复
     * @param code  插件编码
     * @return
     */
    public boolean existsSameCode(String code) {
        String sql = "select count(1) from plugins where code = :code and is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        return this.countQuery(sql, paramMap).intValue() > 0;
    }

    /**
     * 修改插件状态
     * @param id     插件id
     * @param status 插件状态
     * @return
     */
    public boolean updateStatus(Long id, Boolean status) {
        String sql = "update plugins set status = :status, gmt_modify = :gmtModify where id = :id and is_deleted = 0";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        paramMap.put("gmtModify", new Date());
        paramMap.put("status", status);
        return this.execute(sql, paramMap) > 0;
    }
}
